@@ -1,14 +1,16 @@
 {-# LANGUAGE FlexibleContexts #-}
 -- | Use lazy I\/O for consuming the contents of a source. Warning: All normal
--- warnings of lazy I\/O apply. However, if you consume the content within the
--- ResourceT, you should be safe.
+-- warnings of lazy I\/O apply. In particular, if you are using this with a
+-- @ResourceT@ transformer, you must force the list to be evaluated before
+-- exiting the @ResourceT@.
 module Data.Conduit.Lazy
     ( lazyConsume
     ) where
 
 import Data.Conduit
+import Data.Conduit.Internal (Pipe (..))
 import System.IO.Unsafe (unsafeInterleaveIO)
-import Control.Monad.Trans.Control (MonadBaseControl, liftBaseOp_)
+import Control.Monad.Trans.Control (liftBaseOp_)
 import Control.Monad.Trans.Resource (MonadActive (monadActive))
 
 -- | Use lazy I\/O to consume all elements from a @Source@.
@@ -17,14 +19,15 @@ import Control.Monad.Trans.Resource (MonadActive (monadActive))
 -- state has been closed.
 --
 -- Since 0.3.0
-lazyConsume :: (MonadBaseControl IO m, MonadActive m) => Source m a -> m [a]
-lazyConsume (Done _ ()) = return []
+lazyConsume :: (MonadBaseControl IO m, MonadActive m) => Pipe l i a () m r -> m [a]
+lazyConsume (Done _) = return []
 lazyConsume (HaveOutput src _ x) = do
     xs <- lazyConsume src
     return $ x : xs
-lazyConsume (PipeM msrc _) = liftBaseOp_ unsafeInterleaveIO $ do
+lazyConsume (PipeM msrc) = liftBaseOp_ unsafeInterleaveIO $ do
     a <- monadActive
     if a
         then msrc >>= lazyConsume
         else return []
-lazyConsume (NeedInput _ c) = lazyConsume c
+lazyConsume (NeedInput _ c) = lazyConsume (c ())
+lazyConsume (Leftover p _) = lazyConsume p
